/**********************************************************************
 * Separate words into distinct files according to length
 * 
 * $ java-algs4 FilterByLength Pwords
 * $ ls -al Pwo* | head -n7
 * -rw-r--r--  1 yoshi  staff  3077701 19 Nov  2016 Pwords
 * -rw-r--r--  1 yoshi  staff       21  1 Apr 23:22 Pwords01.words
 * -rw-r--r--  1 yoshi  staff      295  1 Apr 23:22 Pwords02.words
 * -rw-r--r--  1 yoshi  staff     1675  1 Apr 23:22 Pwords03.words
 * -rw-r--r--  1 yoshi  staff    10813  1 Apr 23:22 Pwords04.words
 * -rw-r--r--  1 yoshi  staff    40366  1 Apr 23:22 Pwords05.words
 * -rw-r--r--  1 yoshi  staff    95155  1 Apr 23:22 Pwords06.words
 * $ head Pwords05.words
 * Aarão
 * Abade
 * Abaré
 * Abdon
 * Abeba
 * Abner
 * Abreu
 * Abuja
 * Acaia
 * Acari
 * $
 *
 * Can also do things such as
 * 
 * $ java-algs4 FilterByLength < DATA/leipzig100k.txt
 * 
 **********************************************************************/

import edu.princeton.cs.algs4.In;
import edu.princeton.cs.algs4.Out;
import edu.princeton.cs.algs4.LinearProbingHashST;
import edu.princeton.cs.algs4.SET;

public class FilterByLength {

    private static String pad(int l) {
	if (l < 10) return "0" + l;
	else return "" + l;
    }

    public static void main(String[] args) {
	In in; 
	if (args.length > 0) in = new In(args[0]);
	else in = new In();  // read words from stdin

	Words w = new Words(in);

	LinearProbingHashST<Integer, SET<String>> st = new LinearProbingHashST<Integer, SET<String>>();

	for (String word = w.nextWord(); word != null; word = w.nextWord()) {
	    int l = word.length();
	    if (!st.contains(l)) st.put(l, new SET<String>());
	    st.get(l).add(word);
	}

	for (int l : st.keys()) {
	    String pref;
	    if (args.length > 0) pref = args[0];
	    else pref = "stdout";
	    Out out = new Out(pref + pad(l) + ".words");
	    for (String s : st.get(l)) out.println(s);
	}
    }
}
