/**
 * $ java-algs4 WordComponentsPlain < DATA/5+6_letter_words/words5.txt | head -n8
 * Time to produce wordgraph: 0.472
 * 2415 vertices and 2740 edges
 * Ave degree: 2.269151138716356
 * 758 components
 * aback 
 * abase abash abate abuse agate agave amuse awash 
 * abbas 
 * abbey 
 * abbot 
 * abide abode above amide anode aside 
 * abort about amort 
 * abyss 
 * $ 
 */

import edu.princeton.cs.algs4.In;
import edu.princeton.cs.algs4.Graph;
import edu.princeton.cs.algs4.CC;
import edu.princeton.cs.algs4.Queue;
import edu.princeton.cs.algs4.StdOut;
import edu.princeton.cs.algs4.Stopwatch;

public class WordComponentsPlain
{
    public static void main(String[] args)
    {
	In in; 
	if (args.length == 0) in = new In();
	else in = new In(args[0]);
	Stopwatch sw = new Stopwatch();
        WordGraphPlain wg = new WordGraphPlain(in);
	System.err.println("Time to produce wordgraph: " + sw.elapsedTime());
        Graph G = wg.graph();

	System.err.println(G.V() + " vertices and " + G.E() + " edges");
	System.err.println("Ave degree: " + 2.0*G.E()/G.V());

        CC cc = new CC(G);

        // number of connected components
        int m = cc.count();
        System.err.println(m + " components");

        // compute list of vertices in each connected component
        Queue<Integer>[] components = (Queue<Integer>[]) new Queue[m];
        for (int i = 0; i < m; i++) 
            components[i] = new Queue<Integer>();

        for (int v = 0; v < G.V(); v++) 
            components[cc.id(v)].enqueue(v);

        // print results
        for (int i = 0; i < m; i++) {
            for (int v : components[i]) 
                StdOut.print(wg.nameOf(v) + " ");		
            StdOut.println();
        }
    }
}
