/**
 * $ cat DATA/5+6_letter_words/words6.txt | java-algs4 WordGraphPlain | head
 * Time to build word graph: 0.614
 * abacus
 * abater
 * abduct
 * abject
 *    object
 *    adject
 * ablate
 *    oblate
 *    ablaze
 * $ 
 */

import edu.princeton.cs.algs4.Graph;
import edu.princeton.cs.algs4.In;
import edu.princeton.cs.algs4.StdIn;
import edu.princeton.cs.algs4.StdOut;
import edu.princeton.cs.algs4.Stopwatch;

/**
 * Based on WordLadder.java
 */

public class WordGraphPlain {
    private Graph graph;             // the underlying graph
    private IndexSET<String> st;     // string <-> index
    private int wordLength;          // all words have this length
    
    public WordGraphPlain(In in) {
        st = new IndexSET<>();
        // First pass builds the index by reading strings to associate
        // distinct strings with an index
	String[] a = in.readAllStrings();
	wordLength = a[0].length();
	for (int i = 0; i < a.length; i++) {
	    if (!st.contains(a[i]))
		st.add(a[i]);
	    if (a[i].length() != wordLength) 
		throw new RuntimeException("Words of different lengths not allowed in list");
	}

        // second pass builds the graph
        graph = new Graph(st.size());
	for (int i = 0; i < st.size(); i++) {
	    String word1 = st.keyOf(i);
	    for (int j = i + 1; j < st.size(); j++) {
		String word2 = st.keyOf(j);
		if (isNeighbor(word1, word2)) 
		    graph.addEdge(st.indexOf(word1), st.indexOf(word2));
	    }
	}
    }

    // return true if two strings differ in exactly one letter
    // or the shorter string can be obtained from the longer string
    // deleting exactly one letter
    public static boolean isNeighbor(String a, String b) {
	if (a.length() == b.length()) { 
	    int differ = 0;
	    for (int i = 0; i < a.length(); i++) {
		if (a.charAt(i) != b.charAt(i)) differ++;
		if (differ > 1) return false;
	    }
	    return true;
	}
	return false;
    }

    public boolean contains(String s) {
        return st.contains(s);
    }

    public int indexOf(String s) {
        return st.indexOf(s);
    }

    public String nameOf(int v) {
        validateVertex(v);
        return st.keyOf(v);
    }

    public Graph graph() {
        return graph;
    }

    // throw an IllegalArgumentException unless {@code 0 <= v < V}
    private void validateVertex(int v) {
        int V = graph.V();
        if (v < 0 || v >= V)
            throw new IllegalArgumentException("vertex " + v + " is not between 0 and " + (V - 1));
    }

    public static void main(String[] args) {
        In in;
	if (args.length > 0) in = new In(args[0]);
	else in = new In();
	Stopwatch sw = new Stopwatch();
        WordGraphPlain wg = new WordGraphPlain(in);
	StdOut.println("Time to build word graph: " + sw.elapsedTime());
        Graph graph = wg.graph();
	for (int i = 0; i < graph.V(); i++) {
	    StdOut.println(wg.nameOf(i));
	    for (int j : graph.adj(i)) {
		StdOut.println("   " + wg.nameOf(j));
	    }
        }
    }
}
